#!/bin/bash
set +m
HERE=$(pwd)
MY_DIR=$(cd $(dirname $0); pwd)
CMD="$1"

function Usage()
{
	echo "Usage:"
	echo "  $0 run <target> [<max parallel jobs>]"
	echo "  $0 sim <target> <simulator> [<max parallel jobs>]"
	echo "  $0 compare <target1> <run | sim> <target2> <run | sim>"
	echo "  $0 check <target>"
}

MAX_PARALLEL_PIDS=1
PID_QUEUE=""
PID_QUEUE_LENGTH=0

case "${CMD}" in
	run)
		TARGET="$2"
		if [ -n "$3" ]; then
			MAX_PARALLEL_PIDS="$3"
		fi
		if [ -z "${TARGET}" ]; then
			Usage
			exit -1
		fi
		BUILD_DIR="${MY_DIR}/builds/${TARGET}"
		;;
	sim)
		TARGET="$2"
		SIMULATOR="$3"
		if [ -n "$4" ]; then
			MAX_PARALLEL_PIDS="$4"
		fi
		if [ -z "${TARGET}" ] || [ -z "${SIMULATOR}" ]; then
			Usage
			exit -1
		fi
		BUILD_DIR="${MY_DIR}/builds/${TARGET}"
		;;
	compare)
		TARGET1="$2"
		PLATFORM1="$3"
		TARGET2="$4"
		PLATFORM2="$5"
		if [ -z "${TARGET1}" ] || [ -z "${PLATFORM1}" ] || [ -z "${TARGET2}" ] || [ -z "${PLATFORM2}" ]; then
			Usage
			exit -1
		fi
		BUILD_DIR1="${MY_DIR}/builds/${TARGET1}"
		BUILD_DIR2="${MY_DIR}/builds/${TARGET2}"
		;;
	check)
		TARGET="$2"
		if [ -z "${TARGET}" ]; then
			Usage
			exit -1
		fi
		
		case "${TARGET}" in
			powerpc-440fp*)
				REF_TARGET="powerpc-440fp";
				;;
			powerpc-7450*)
				REF_TARGET="powerpc-7450";
				;;
			arm*)
				REF_TARGET="arm";
				;;
			avr32*)
				REF_TARGET="avr32";
				;;
			*)
				REF_TARGET="${TARGET}"
				;;
		esac
		REF_DIR="${MY_DIR}/refs/${REF_TARGET}"
		BUILD_DIR="${MY_DIR}/builds/${TARGET}"
		;;
	*)
		Usage
		exit -1
esac

function QueuePID()
{
	PID_QUEUE="${PID_QUEUE} $1"
	PID_QUEUE_LENGTH=$((${PID_QUEUE_LENGTH}+1))
}

function DequeuePID()
{
	local OLD_PID_QUEUE=${PID_QUEUE}
	PID_QUEUE=""
	for PID in ${OLD_PID_QUEUE}; do
		if [ ! "${PID}" = "$1" ] ; then
			PID_QUEUE="${PID_QUEUE} ${PID}"
		fi
	done
	PID_QUEUE_LENGTH=$(($PID_QUEUE_LENGTH-1))
}

function CheckPIDQueue()
{
	jobs >/dev/null 2>/dev/null    # strange: this make sure CHILD_PIDS gets updated
	local OLD_PID_QUEUE=${PID_QUEUE}
	local CHILD_PIDS=$(jobs -l | sed -n 's/^ *\(\[[0-9]*\]\)\? *\(+\|-\)\? *\([0-9]*\) .*$/\3 /p')
	for PID in ${OLD_PID_QUEUE}; do
		PID_STILL_RUNNING=0
		for CHILD_PID in ${CHILD_PIDS}; do
			if [ ${PID} -eq ${CHILD_PID} ]; then
				PID_STILL_RUNNING=1
				break
			fi
		done
		if [ ${PID_STILL_RUNNING} -eq 0 ]; then
			DequeuePID ${PID}
		fi
	done
}

function WaitUntilReadyToQueuePID()
{
	while [ ${PID_QUEUE_LENGTH} -ge ${MAX_PARALLEL_PIDS} ]; do
		CheckPIDQueue
		sleep 1
	done
}

function HexDump()
{
	hexdump -v -e '"%08_ax "' -e '32/1 "%.2x"' -e '"\n"' "$1"
}

function Run()
{
	WaitUntilReadyToQueuePID

	local BENCH_DIR="$1"
	local RUN_CMD="$2"
	local FLAVOR="$3"
	
	local HERE=$(pwd)
	cd "${BUILD_DIR}/${BENCH_DIR}"
	local BENCH_NAME=$(printf "${BENCH_DIR}" | sed -e 's/\// /g')
	printf "Running ${BENCH_NAME}"
	if [ -n "${FLAVOR}" ]; then
		printf " with ${FLAVOR} dataset"
	fi
	printf " for ${TARGET}...\n"
	local PID
	if [ -n "${FLAVOR}" ]; then
		( PATH="$(pwd):${PATH}" eval "${RUN_CMD}" ) 2> /dev/null | sed 's/\x0D$//' > output_${FLAVOR}.run.txt &
		PID=$!
	else
		( PATH="$(pwd):${PATH}" eval "${RUN_CMD}" ) 2> /dev/null | sed 's/\x0D$//' > output.run.txt &
		PID=$!
	fi
	QueuePID ${PID}
	cd "${HERE}"
}

function Sim()
{
	WaitUntilReadyToQueuePID

	local BENCH_DIR="$1"
	local SIM_CMD="$2"
	local FLAVOR="$3"
	
	local HERE=$(pwd)
	cd "${BUILD_DIR}/${BENCH_DIR}"
	local BENCH_NAME=$(printf "${BENCH_DIR}" | sed -e 's/\// /g')
	printf "Simulating ${BENCH_NAME}"
	if [ -n "${FLAVOR}" ]; then
		printf " with ${FLAVOR} dataset"
	fi
	printf " for ${TARGET}...\n"
	local PID
	if [ -n "${FLAVOR}" ]; then
		( PATH="$(pwd):${PATH}" SIMULATE="${SIMULATOR}" eval "${SIM_CMD}" ) 2> sim_out_${FLAVOR}.txt | sed 's/\x0D$//' > output_${FLAVOR}.sim.txt &
		PID=$!
	else
		( PATH="$(pwd):${PATH}" SIMULATE="${SIMULATOR}" eval "${SIM_CMD}" ) 2> sim_out.txt | sed 's/\x0D$//' > output.sim.txt &
		PID=$!
	fi
	QueuePID ${PID}
	cd "${HERE}"
}

function Compare()
{
	BENCH_DIR="$1"
	FLAVOR="$2"

	local FILENAME=$(printf "${BENCH_DIR}" | sed -e 's/\//_/g' -e 's/\(.*\)/\L\1/')
	local BENCH_NAME=$(printf "${BENCH_DIR}" | sed -e 's/\// /g')
	local CONSOLE_OUT
	printf "${BENCH_NAME}";
	if [ -n "${FLAVOR}" ]; then
		printf " ${FLAVOR}"
	fi
	printf ":";
	if [ -n "${FLAVOR}" ]; then
		diff -Naur "${BUILD_DIR1}/${BENCH_DIR}/output_${FLAVOR}.${PLATFORM1}.txt" "${BUILD_DIR2}/${BENCH_DIR}/output_${FLAVOR}.${PLATFORM2}.txt" > "${FILENAME}_${FLAVOR}_${TARGET1}-${PLATFORM1}_${TARGET2}-${PLATFORM2}.diff"
	else
		diff -Naur "${BUILD_DIR1}/${BENCH_DIR}/output.${PLATFORM1}.txt" "${BUILD_DIR2}/${BENCH_DIR}/output.${PLATFORM2}.txt" > "${FILENAME}_${TARGET1}-${PLATFORM1}_${TARGET2}-${PLATFORM2}.diff"
	fi
	if [ $? -eq 0 ]; then
		printf 'pass\n'
	else
		printf 'fail\n'
	fi
}

function Check()
{
	BENCH_DIR="$1"
	FLAVOR="$2"

	local FILENAME=$(printf "${BENCH_DIR}" | sed -e 's/\//_/g' -e 's/\(.*\)/\L\1/')
	local BENCH_NAME=$(printf "${BENCH_DIR}" | sed -e 's/\// /g')
	local CONSOLE_OUT
	printf "${BENCH_NAME}";
	if [ -n "${FLAVOR}" ]; then
		printf " ${FLAVOR}"
	fi
	printf ":";
	if [ -n "${FLAVOR}" ]; then
		diff -Naur "${REF_DIR}/${BENCH_DIR}/output_${FLAVOR}.ref.txt" "${BUILD_DIR}/${BENCH_DIR}/output_${FLAVOR}.sim.txt" > "${FILENAME}_${FLAVOR}_${TARGET}.diff"
	else
		diff -Naur "${REF_DIR}/${BENCH_DIR}/output.ref.txt" "${BUILD_DIR}/${BENCH_DIR}/output.sim.txt" > "${FILENAME}_${FLAVOR}_${TARGET}.diff"
	fi
	if [ $? -eq 0 ]; then
		printf 'pass\n'
	else
		printf 'fail\n'
	fi
}

case "${CMD}" in
	run)
		Run 'automotive/basicmath' 'runme_large.sh; cat output_large.txt' 'large'
		Run 'automotive/basicmath' 'runme_small.sh; cat output_small.txt' 'small'
		Run 'automotive/bitcount' 'runme_large.sh; cat output_large.txt | sed -e '\''s/^\(.*\)Time: *\(+\|-\)\?[0-9][0-9]*\.[0-9][0-9]* sec\(.*\)$/\1Time: \*.\*\*\* sec\3/g'\'' -e '\''s///g'\'' -e '\''s/^Best  >.*$/Best  > N\/A/'\'' -e '\''s/^Worst >.*$/Worst > N\/A/'\''' 'large'
		Run 'automotive/bitcount' 'runme_small.sh; cat output_small.txt | sed -e '\''s/^\(.*\)Time: *\(+\|-\)\?[0-9][0-9]*\.[0-9][0-9]* sec\(.*\)$/\1Time: \*.\*\*\* sec\3/g'\'' -e '\''s///g'\'' -e '\''s/^Best  >.*$/Best  > N\/A/'\'' -e '\''s/^Worst >.*$/Worst > N\/A/'\''' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Run 'automotive/qsort' 'runme_large.sh; cat output_large.txt' 'large'
				Run 'automotive/qsort' 'runme_small.sh; cat output_small.txt' 'small'
				Run 'automotive/susan' 'runme_large.sh; HexDump output_large.smoothing.pgm; HexDump output_large.edges.pgm; HexDump output_large.corners.pgm' 'large'
				Run 'automotive/susan' 'runme_small.sh; HexDump output_small.smoothing.pgm; HexDump output_small.edges.pgm; HexDump output_small.corners.pgm' 'small'
				Run 'consumer/jpeg' 'runme_large.sh; HexDump output_large_encode.jpeg; HexDump output_large_decode.ppm' 'large'
				Run 'consumer/jpeg' 'runme_small.sh; HexDump output_small_encode.jpeg; HexDump output_small_decode.ppm' 'small'
				Run 'consumer/lame' 'runme_large.sh &> /dev/null; HexDump output_large.mp3' 'large'
				Run 'consumer/lame' 'runme_small.sh &> /dev/null; HexDump output_small.mp3' 'small'
				Run 'consumer/typeset' 'runme_large.sh; cat output_large.ps | sed -e '\''s/^%%CreationDate:.*$/%%CreationDate: Thu Jan  1 00:00:00 1970/'\''' 'large'
				Run 'consumer/typeset' 'runme_small.sh; cat output_small.ps | sed -e '\''s/^%%CreationDate:.*$/%%CreationDate: Thu Jan  1 00:00:00 1970/'\''' 'small'
				Run 'consumer/mad' 'runme_large.sh; HexDump output_large.wav' 'large'
				Run 'consumer/mad' 'runme_small.sh; HexDump output_small.wav' 'small'
				Run 'consumer/tiffmedian' 'runme_large.sh; HexDump output_largemedian.tif' 'large'
				Run 'consumer/tiffmedian' 'runme_small.sh; HexDump output_smallmedian.tif' 'small'
				Run 'consumer/tiffdither' 'runme_large.sh; HexDump output_largedither.tif' 'large'
				Run 'consumer/tiffdither' 'runme_small.sh; HexDump output_smalldither.tif' 'small'
				Run 'consumer/tiff2rgba' 'runme_large.sh; HexDump output_largergba.tif' 'large'
				Run 'consumer/tiff2rgba' 'runme_small.sh; HexDump output_smallrgba.tif' 'small'
				Run 'consumer/tiff2bw' 'runme_large.sh; HexDump output_largebw.tif' 'large'
				Run 'consumer/tiff2bw' 'runme_small.sh; HexDump output_smallbw.tif' 'small'
				;;
		esac
		Run 'network/dijkstra' 'runme_large.sh; cat output_large.dat' 'large'
		Run 'network/dijkstra' 'runme_small.sh; cat output_small.dat' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Run 'network/patricia' 'runme_large.sh; cat output_large.txt' 'large'
				Run 'network/patricia' 'runme_small.sh; cat output_small.txt' 'small'
				;;
		esac
		case "${TARGET}" in
			*86*)
				Run 'office/ghostscript' 'runme_large.sh; HexDump data/output_large.ppm' 'large'
				Run 'office/ghostscript' 'runme_small.sh; HexDump data/output_small.ppm' 'small'
		esac
		case "${TARGET}" in
			arm* | *86*)
				Run 'office/ispell' 'runme_large.sh; cat tests/output_large.txt' 'large'
				Run 'office/ispell' 'runme_small.sh; cat tests/output_small.txt' 'small'
				;;
		esac
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Run 'office/rsynth' 'runme_large.sh; HexDump large_output.au' 'large'
				Run 'office/rsynth' 'runme_small.sh; HexDump small_output.au' 'small'
				;;
		esac
		Run 'office/stringsearch' 'runme_large.sh; cat output_large.txt' 'large'
		Run 'office/stringsearch' 'runme_small.sh; cat output_small.txt' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Run 'office/sphinx' 'runme.sh | sed -e '\''s/  *[0-9][0-9]*\.[0-9][0-9]* sec/ **\.** sec/g'\'' -e '\''s/  *[0-9][0-9]*\.[0-9][0-9]* xRT/ **\.** xRT/g'\'''
				;;
		esac
		Run 'security/blowfish' 'runme_large.sh; HexDump output_large.enc; HexDump output_large.asc' 'large'
		Run 'security/blowfish' 'runme_small.sh; HexDump output_small.enc; HexDump output_small.asc' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Run 'security/pgp' 'runme.sh > /dev/null; cat tests/testin.txt.asc; cat tests/testout.txt.asc'
				;;
		esac
		Run 'security/rijndael' 'runme_large.sh; HexDump output_large.enc; cat output_large.dec' 'large'
		Run 'security/rijndael' 'runme_small.sh; HexDump output_small.enc; cat output_small.dec' 'small'
		Run 'security/sha' 'runme_large.sh; cat output_large.txt' 'large'
		Run 'security/sha' 'runme_small.sh; cat output_small.txt' 'small'
		Run 'telecomm/adpcm' 'runme_large.sh; HexDump output_large.adpcm; HexDump output_large.pcm' 'large'
		Run 'telecomm/adpcm' 'runme_small.sh; HexDump output_small.adpcm; HexDump output_small.pcm' 'small'
		Run 'telecomm/CRC32' 'runme_large.sh; cat output_large.txt' 'large'
		Run 'telecomm/CRC32' 'runme_small.sh; cat output_small.txt' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Run 'telecomm/FFT' 'runme_large.sh; cat output_large.txt; cat output_large.inv.txt' 'large'
				Run 'telecomm/FFT' 'runme_small.sh; cat output_small.txt; cat output_small.inv.txt' 'small'
				Run 'telecomm/gsm' 'runme_large.sh; HexDump output_large.encode.gsm; HexDump output_large.decode.run' 'large'
				Run 'telecomm/gsm' 'runme_small.sh; HexDump output_small.encode.gsm; HexDump output_small.decode.run' 'small'
				;;
		esac
		wait
		;;
	sim)
		Sim 'automotive/basicmath' 'simme_large.sh; cat output_large.txt' 'large'
		Sim 'automotive/basicmath' 'simme_small.sh; cat output_small.txt' 'small'
		Sim 'automotive/bitcount' 'simme_large.sh; cat output_large.txt | sed -e '\''s/^\(.*\)Time: *\(+\|-\)\?[0-9][0-9]*\.[0-9][0-9]* sec\(.*\)$/\1Time: \*.\*\*\* sec\3/g'\'' -e '\''s///g'\'' -e '\''s/^Best  >.*$/Best  > N\/A/'\'' -e '\''s/^Worst >.*$/Worst > N\/A/'\''' 'large'
		Sim 'automotive/bitcount' 'simme_small.sh; cat output_small.txt | sed -e '\''s/^\(.*\)Time: *\(+\|-\)\?[0-9][0-9]*\.[0-9][0-9]* sec\(.*\)$/\1Time: \*.\*\*\* sec\3/g'\'' -e '\''s///g'\'' -e '\''s/^Best  >.*$/Best  > N\/A/'\'' -e '\''s/^Worst >.*$/Worst > N\/A/'\''' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Sim 'automotive/qsort' 'simme_large.sh; cat output_large.txt' 'large'
				Sim 'automotive/qsort' 'simme_small.sh; cat output_small.txt' 'small'
				Sim 'automotive/susan' 'simme_large.sh; HexDump output_large.smoothing.pgm; HexDump output_large.edges.pgm; HexDump output_large.corners.pgm' 'large'
				Sim 'automotive/susan' 'simme_small.sh; HexDump output_small.smoothing.pgm; HexDump output_small.edges.pgm; HexDump output_small.corners.pgm' 'small'
				Sim 'consumer/jpeg' 'simme_large.sh; HexDump output_large_encode.jpeg; HexDump output_large_decode.ppm' 'large'
				Sim 'consumer/jpeg' 'simme_small.sh; HexDump output_small_encode.jpeg; HexDump output_small_decode.ppm' 'small'
				Sim 'consumer/lame' 'simme_large.sh &> /dev/null; HexDump output_large.mp3' 'large'
				Sim 'consumer/lame' 'simme_small.sh &> /dev/null; HexDump output_small.mp3' 'small'
				Sim 'consumer/typeset' 'simme_large.sh; cat output_large.ps | sed -e '\''s/^%%CreationDate:.*$/%%CreationDate: Thu Jan  1 00:00:00 1970/'\''' 'large'
				Sim 'consumer/typeset' 'simme_small.sh; cat output_small.ps | sed -e '\''s/^%%CreationDate:.*$/%%CreationDate: Thu Jan  1 00:00:00 1970/'\''' 'small'
				Sim 'consumer/mad' 'simme_large.sh; HexDump output_large.wav' 'large'
				Sim 'consumer/mad' 'simme_small.sh; HexDump output_small.wav' 'small'
				Sim 'consumer/tiffmedian' 'simme_large.sh; HexDump output_largemedian.tif' 'large'
				Sim 'consumer/tiffmedian' 'simme_small.sh; HexDump output_smallmedian.tif' 'small'
				Sim 'consumer/tiffdither' 'simme_large.sh; HexDump output_largedither.tif' 'large'
				Sim 'consumer/tiffdither' 'simme_small.sh; HexDump output_smalldither.tif' 'small'
				Sim 'consumer/tiff2rgba' 'simme_large.sh; HexDump output_largergba.tif' 'large'
				Sim 'consumer/tiff2rgba' 'simme_small.sh; HexDump output_smallrgba.tif' 'small'
				Sim 'consumer/tiff2bw' 'simme_large.sh; HexDump output_largebw.tif' 'large'
				Sim 'consumer/tiff2bw' 'simme_small.sh; HexDump output_smallbw.tif' 'small'
				;;
		esac
		Sim 'network/dijkstra' 'simme_large.sh; cat output_large.dat' 'large'
		Sim 'network/dijkstra' 'simme_small.sh; cat output_small.dat' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Sim 'network/patricia' 'simme_large.sh; cat output_large.txt' 'large'
				Sim 'network/patricia' 'simme_small.sh; cat output_small.txt' 'small'
				;;
		esac
		case "${TARGET}" in
			*86*)
				Sim 'office/ghostscript' 'simme_large.sh; HexDump data/output_large.ppm' 'large'
				Sim 'office/ghostscript' 'simme_small.sh; HexDump data/output_small.ppm' 'small'
				;;
		esac
		case "${TARGET}" in
			arm* | *86*)
				Sim 'office/ispell' 'simme_large.sh; cat tests/output_large.txt' 'large'
				Sim 'office/ispell' 'simme_small.sh; cat tests/output_small.txt' 'small'
				;;
		esac
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Sim 'office/rsynth' 'simme_large.sh; HexDump large_output.au' 'large'
				Sim 'office/rsynth' 'simme_small.sh; HexDump small_output.au' 'small'
				;;
		esac
		Sim 'office/stringsearch' 'simme_large.sh; cat output_large.txt' 'large'
		Sim 'office/stringsearch' 'simme_small.sh; cat output_small.txt' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Sim 'office/sphinx' 'simme.sh | sed -e '\''s/  *[0-9][0-9]*\.[0-9][0-9]* sec/ **\.** sec/g'\'' -e '\''s/  *[0-9][0-9]*\.[0-9][0-9]* xRT/ **\.** xRT/g'\'''
				;;
		esac
		Sim 'security/blowfish' 'simme_large.sh; HexDump output_large.enc; HexDump output_large.asc' 'large'
		Sim 'security/blowfish' 'simme_small.sh; HexDump output_small.enc; HexDump output_small.asc' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Sim 'security/pgp' 'simme.sh > /dev/null; cat tests/testin.txt.asc; cat tests/testout.txt.asc'
				;;
		esac
		Sim 'security/rijndael' 'simme_large.sh; HexDump output_large.enc; cat output_large.dec' 'large'
		Sim 'security/rijndael' 'simme_small.sh; HexDump output_small.enc; cat output_small.dec' 'small'
		Sim 'security/sha' 'simme_large.sh; cat output_large.txt' 'large'
		Sim 'security/sha' 'simme_small.sh; cat output_small.txt' 'small'
		Sim 'telecomm/adpcm' 'simme_large.sh; HexDump output_large.adpcm; HexDump output_large.pcm' 'large'
		Sim 'telecomm/adpcm' 'simme_small.sh; HexDump output_small.adpcm; HexDump output_small.pcm' 'small'
		Sim 'telecomm/CRC32' 'simme_large.sh; cat output_large.txt' 'large'
		Sim 'telecomm/CRC32' 'simme_small.sh; cat output_small.txt' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Sim 'telecomm/FFT' 'simme_large.sh; cat output_large.txt; cat output_large.inv.txt' 'large'
				Sim 'telecomm/FFT' 'simme_small.sh; cat output_small.txt; cat output_small.inv.txt' 'small'
				Sim 'telecomm/gsm' 'simme_large.sh; HexDump output_large.encode.gsm; HexDump output_large.decode.run' 'large'
				Sim 'telecomm/gsm' 'simme_small.sh; HexDump output_small.encode.gsm; HexDump output_small.decode.run' 'small'
				;;
		esac
		wait
		;;
	compare)
		Compare 'automotive/basicmath' 'large'
		Compare 'automotive/basicmath' 'small'
		Compare 'automotive/bitcount' 'large'
		Compare 'automotive/bitcount' 'small'
		case "${TARGET2}" in
			powerpc* | arm* | *86*)
				Compare 'automotive/qsort' 'large'
				Compare 'automotive/qsort' 'small'
				Compare 'automotive/susan' 'large'
				Compare 'automotive/susan' 'small'
				Compare 'consumer/jpeg' 'large'
				Compare 'consumer/jpeg' 'small'
				Compare 'consumer/lame' 'large'
				Compare 'consumer/lame' 'small'
				Compare 'consumer/typeset' 'large'
				Compare 'consumer/typeset' 'small'
				Compare 'consumer/mad' 'large'
				Compare 'consumer/mad' 'small'
				Compare 'consumer/tiffmedian' 'large'
				Compare 'consumer/tiffmedian' 'small'
				Compare 'consumer/tiffdither' 'large'
				Compare 'consumer/tiffdither' 'small'
				Compare 'consumer/tiff2rgba' 'large'
				Compare 'consumer/tiff2rgba' 'small'
				Compare 'consumer/tiff2bw' 'large'
				Compare 'consumer/tiff2bw' 'small'
				;;
		esac
		Compare 'network/dijkstra' 'large'
		Compare 'network/dijkstra' 'small'
		case "${TARGET2}" in
			powerpc* | arm* | *86*)
				Compare 'network/patricia' 'large'
				Compare 'network/patricia' 'small'
				;;
		esac
		case "${TARGET2}" in
			*86*)
				Compare 'office/ghostscript' 'large'
				Compare 'office/ghostscript' 'small'
				;;
		esac
		case "${TARGET2}" in
			arm* | *86*)
				Compare 'office/ispell' 'large'
				Compare 'office/ispell' 'small'
				;;
		esac
		case "${TARGET2}" in
			powerpc* | arm* | *86*)
				Compare 'office/rsynth' 'large'
				Compare 'office/rsynth' 'small'
				;;
		esac
		Compare 'office/stringsearch' 'large'
		Compare 'office/stringsearch' 'small'
		case "${TARGET2}" in
			powerpc* | arm* | *86*)
				Compare 'office/sphinx'
				;;
		esac
		Compare 'security/blowfish' 'large'
		Compare 'security/blowfish' 'small'
		case "${TARGET2}" in
			powerpc* | arm* | *86*)
				Compare 'security/pgp'
				;;
		esac
		Compare 'security/rijndael' 'large'
		Compare 'security/rijndael' 'small'
		Compare 'security/sha' 'large'
		Compare 'security/sha' 'small'
		Compare 'telecomm/adpcm' 'large'
		Compare 'telecomm/adpcm' 'small'
		Compare 'telecomm/CRC32' 'large'
		Compare 'telecomm/CRC32' 'small'
		case "${TARGET2}" in
			powerpc* | arm* | *86*)
				Compare 'telecomm/FFT' 'large'
				Compare 'telecomm/FFT' 'small'
				Compare 'telecomm/gsm' 'large'
				Compare 'telecomm/gsm' 'small'
				;;
		esac
		;;
	check)
		Check 'automotive/basicmath' 'large'
		Check 'automotive/basicmath' 'small'
		Check 'automotive/bitcount' 'large'
		Check 'automotive/bitcount' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Check 'automotive/qsort' 'large'
				Check 'automotive/qsort' 'small'
				Check 'automotive/susan' 'large'
				Check 'automotive/susan' 'small'
				Check 'consumer/jpeg' 'large'
				Check 'consumer/jpeg' 'small'
				Check 'consumer/lame' 'large'
				Check 'consumer/lame' 'small'
				Check 'consumer/typeset' 'large'
				Check 'consumer/typeset' 'small'
				Check 'consumer/mad' 'large'
				Check 'consumer/mad' 'small'
				Check 'consumer/tiffmedian' 'large'
				Check 'consumer/tiffmedian' 'small'
				Check 'consumer/tiffdither' 'large'
				Check 'consumer/tiffdither' 'small'
				Check 'consumer/tiff2rgba' 'large'
				Check 'consumer/tiff2rgba' 'small'
				Check 'consumer/tiff2bw' 'large'
				Check 'consumer/tiff2bw' 'small'
				;;
		esac
		Check 'network/dijkstra' 'large'
		Check 'network/dijkstra' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Check 'network/patricia' 'large'
				Check 'network/patricia' 'small'
				;;
		esac
		case "${TARGET}" in
			*86*)
				Check 'office/ghostscript' 'large'
				Check 'office/ghostscript' 'small'
				;;
		esac
		case "${TARGET}" in
			arm* | *86*)
				Check 'office/ispell' 'large'
				Check 'office/ispell' 'small'
				;;
		esac
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Check 'office/rsynth' 'large'
				Check 'office/rsynth' 'small'
				;;
		esac
		Check 'office/stringsearch' 'large'
		Check 'office/stringsearch' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Check 'office/sphinx'
				;;
		esac
		Check 'security/blowfish' 'large'
		Check 'security/blowfish' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Check 'security/pgp'
				;;
		esac
		Check 'security/rijndael' 'large'
		Check 'security/rijndael' 'small'
		Check 'security/sha' 'large'
		Check 'security/sha' 'small'
		Check 'telecomm/adpcm' 'large'
		Check 'telecomm/adpcm' 'small'
		Check 'telecomm/CRC32' 'large'
		Check 'telecomm/CRC32' 'small'
		case "${TARGET}" in
			powerpc* | arm* | *86*)
				Check 'telecomm/FFT' 'large'
				Check 'telecomm/FFT' 'small'
				Check 'telecomm/gsm' 'large'
				Check 'telecomm/gsm' 'small'
				;;
		esac
		;;
esac







cd ${HERE}

